'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Action = Action;
exports.transform = transform;
exports.default = Reducer;
function Action(type) {
  var transform = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function (p) {
    return p;
  };

  function creator() {
    var payload = transform.apply(undefined, arguments);
    var action = { type: type };
    if (payload !== undefined) {
      action.payload = payload;
    }
    if (payload instanceof Error) {
      action.error = true;
    }
    return action;
  }
  creator.type = type;
  return creator;
}

function transform(creator, transform) {
  var type = creator.type;

  var transformed = function transformed() {
    return creator(transform.apply(undefined, arguments));
  };
  if (type !== undefined) {
    transformed.type = type;
  }
  return transformed;
}

function copy(value) {
  return Array.isArray(value) ? value.slice() : Object.assign({}, value);
}

function Reducer(prefix, initial, handlers) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

  options = Object.assign({
    copy: false
  }, options);
  var reducers = {};

  var reducer = function reducer() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initial;
    var action = arguments[1];
    var type = action.type,
        payload = action.payload;

    if (reducers.hasOwnProperty(type)) {
      state = options.copy ? copy(state) : state;
      return reducers[type](state, payload, action);
    } else {
      return state;
    }
  };

  Object.keys(handlers).forEach(function (key) {
    var handler = handlers[key];
    var type = prefix + '/' + key;
    reducer[key] = Action(type);
    reducers[type] = handler;
  });

  return reducer;
}